// Indirizzo della porta parallela utilizzata per il debug (data)
#define LPT_DAT 0x378
// Indirizzo della porta parallela utilizzata per il debug (status)
#define LPT_STS LPT_DAT+1
// Indirizzo della porta parallela utilizzata per il debug (control)
#define LPT_CNT LPT_DAT+2

#include <linux/init.h>
#include <linux/module.h>
#include <linux/ioport.h>
#include <asm/io.h>

#include <rtai.h>
#include <rtai_sched.h>

#include <linux/errno.h>

MODULE_LICENSE("GPL");

// periodo in nanosecondi
#define PERIODO 250000000

static RT_TASK OndaQuadra_Task;
static RTIME period_ns = PERIODO;
static unsigned char value = 0xff;


static void OndaQuadra_Thread(int t)
{
    while(1)
    {
        if (value == 0xff)
            value = 0x00;
        else
            value = 0xff;

        outb(value, LPT_DAT);
        rt_task_wait_period();
    }
}


static int Parallela_init(void)
{
    RTIME period_count;
    RTIME tick_period;
    int retval;
   
    outb(0x00, LPT_DAT); // Resetto l'uscita
    outb(0x00, LPT_CNT); // Imposto la porta in mod. monodirezionale
    
    // periodic mode
    rt_set_periodic_mode();
    
    period_count = nano2count(period_ns);
    tick_period = start_rt_timer(period_count);
    
    // Creo il thread
    retval = rt_task_init(&OndaQuadra_Task, OndaQuadra_Thread, tick_period, 1024, RT_SCHED_LOWEST_PRIORITY, 0, 0);
    
    if (0 != retval)
    {
        if (-EINVAL == retval)
        {
            /* task structure is already in use */
            printk("periodic task: task structure already in use\n");
        }
        else if (-ENOMEM == retval)
        {
            /* stack could not be allocated */
            printk("periodic task: can't allocate stack\n");
        }
        else
        {
            /* unknown error */
            printk("periodic task: error initializing task structure\n");
        }
        return retval;
    }
    
    // Eseguo il thread
    retval = rt_task_make_periodic(&OndaQuadra_Task, rt_get_time() + tick_period, tick_period);
    
    if (0 != retval)
    {
        if (-EINVAL == retval)
        {
            /* task structure is already in use */
            printk("periodic task: task structure is invalid\n");
        }
        else
        {
            /* unknown error */
            printk("periodic task: error starting task\n");
        }
        return retval;
    }
    
    printk(KERN_ALERT "Parallela loaded\n");
    return 0;
}


static void Parallela_exit(void)
{
    outb(0x00, LPT_DAT); // Azzero l'uscita
    outb(0x00, LPT_CNT); // Torno in monodirezionale e spengo P16
    stop_rt_timer();
    rt_task_delete(&OndaQuadra_Task);
    printk(KERN_ALERT "Parallela unloaded\n");
}

module_init(Parallela_init);
module_exit(Parallela_exit);
